// Open-GL Demo "Baureihe 01"
// Uwe Domaratius 2002/03

// Benoetigte Bibliotheken
#include <GL/glut.h>
#include <GL/glaux.h>
// unter Linux <GL/glaux.h> durch "miniaux.h" ersetzen (enthaelt nur den Code zum Texturen laden)!
//#include "miniaux.h"
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#ifndef	M_PI				// PI als Konstante
#define M_PI 3.14159265358979323846
#endif
#ifndef MIN					// Min-Funktion
#define MIN(a,b) ((a) > (b))? (b) : (a)
#endif
#ifndef MAX					// Max-Funktion
#define MAX(a,b) ((a) > (b))? (a) : (b)
#endif

// Parameter zum Verhindern von Clippingfehlern in OpenGL <1.2
#ifndef GL_CLAMP_TO_EDGE	
#define GL_CLAMP_TO_EDGE 0x812F
#endif

// uebliche Bool-Werte definieren
#define TRUE 1
#define FALSE 0

// Die Startposition des Fensters (oben links in der Ecke)
#define WIN_POS_X	0
#define WIN_POS_Y	0

// Breite und Hoehe des Fensters (bei alten Karten beschraenken auf 800*600)
#define WIN_WIDTH	1024
#define WIN_HEIGHT	768

// OpenGL-Modus RGBA, Z-Buffer, Double Buffering
#define USED_MODUS	GLUT_RGBA |  GLUT_DEPTH | GLUT_DOUBLE

// Array fuer die Texturen (fuer insgesamt 5 verschiedene)
GLuint textures[4];

// Name des Programms (erscheint in der Titelleiste)
#define PROG_NAME	"Baureihe 01"

/////////////////////////////////////////////////////////////
// Grundlegende Datenstrukturen und Variablen
/////////////////////////////////////////////////////////////


//! Struktur fuer Bearbeitung der Mausevents
struct MouseStruct {
	int LastState;
	int OldX;
	int OldY;
	int ScreenX;
	int ScreenY;
	int MoveX;
	int MoveY;
	double Radius;
} globMouse;

int MainWin;		// Identifier vom Hauptfenster


/////////////////////////////////////////////////////////////
//	Funktionen und Prozeduren
/////////////////////////////////////////////////////////////
void MenuFuncFile (int);
void DisplayFunc ();
void ReshapeFunc (int,int);
void MouseFunc (int,int,int,int);
void MouseMove (int,int);
void KeyboardFunc (unsigned char,int,int);
void SpecialKeyboardFunc (int,int,int);
void IdleFunc ();
void OnTimer(int);
void Lok ();

// Variablen, teilweise beeinflussbar
double timediff=0;	// Zeit zum Erzeugen eines kompletten Bildes
double v=0.0;		// Geschwindigkeit der Lok
double angle=0.0;	// aktueller Drehwinkel der Raeder
int fps=0;			// momentane Framerate (gerundet)
double fill=0;		// Fuellstand des Tenders
double time1,time2; // Zeitnahmevariablen

// Globale verstellbare Parameter
int shiftx=0;		// Verschieben der Lok links/rechts
int shiftz=0;		// Verschieben der Lok vor/zurueck
int show[11]={1,1,1,1,1,1,1,1,1,1,1};  // Anzeige bestimmter Teile der Lok

// Parameter, mit denen die Darstellungsgeschwindigkeit beeinflusst werden kann
int slices=50;		// Detailgrad von Zylindern und Disks
int mindetail=10;	// Minimaler Detaillevel, der nicht unterschritten wird
int minfps=25;		// bei weniger FPS werden Details verringert
int maxfps=40;		// bei mehr FPS werden mehr Details gezeigt
int size=30;		// Groesse des "Umgebungswuerfels"
double scale=0.25;		// Groesse der Rasenteile

// Switches fuer verschiedene Optionen, 1 bedeutet an, 0 ist aus
int lighting=1;		// Beleuchtung an/aus
int shading=1;		// Smooth/Flat-Shading
int wireframe=0;	// Darstellung Wireframe
int ortho=0;		// Darstellung Parallelprojektion
int autodetail=1;	// automatische Detailanpassung
int light=0;		// Licht an der Lok an/aus
int depth=1;		// Z-Buffering an/aus
int clip=0;			// Clipping-Plane an/aus
int light0=1;		// Sonne an/aus
int light1=0;		// Licht von der Kamera an/aus

// globale Farbdefinitionen
GLubyte schwarz[] ={15,15,15};
GLubyte rot[]     ={255,0,0};
GLubyte grau[]    ={80,80,80};
GLubyte weiss[]   ={255,255,255};
GLfloat window[]  ={0.3,0.3,0.35,0.8};

// Lichtdefinitionen
GLfloat global_ambient_light[]={0.075,0.075,0.075,0.5};

// direktionales Licht (Sonne)
double sunx=1.0;
double suny=1.0;
double sunz=1.0;
double sun_intensity=1.0;

// 6 Spotlights mit verschiedenen Einstellungen an der Lok
double headlight_intensity=1;
double headlight_attenuation=0.001;
double headlight_cutoff=20;

GLfloat headlight_diffuse[]      ={headlight_intensity,headlight_intensity,headlight_intensity,1};
GLfloat headlight_red_diffuse[]  ={headlight_intensity,0,0,1};
GLfloat headlight_specular[]     ={headlight_intensity,headlight_intensity,headlight_intensity,1};
GLfloat headlight_red_specular[] ={headlight_intensity,0,0,1};
GLfloat headlight1_direction[]   ={0,0,1};
GLfloat headlight2_direction[]   ={0,0,-1};
GLfloat headlight_position[]     ={0,0,0.1,1};

// ein Punktlicht an der Kameraposition
double camera_intensity=0.2;
GLfloat camera_light_diffuse[]  ={camera_intensity,camera_intensity,camera_intensity,1};
GLfloat camera_light_specular[] ={camera_intensity,camera_intensity,camera_intensity,1};
GLfloat camera_light_position[] ={0,0,0,1};

// Materialdefinitionen
GLfloat mat_color[]      ={1,1,1,1};
GLfloat mat_emission[]   ={0,0,0,1};
GLfloat mat_shininess[]  ={50};
GLfloat rauch_specular[] ={0,0,0,1};

///////////////////////////////////////////////////////////////////////
// Texturen laden
///////////////////////////////////////////////////////////////////////
AUX_RGBImageRec *LoadBMP(char *Filename) {
	FILE *File=NULL;
	File=fopen(Filename,"r");
	if(File) {
		fclose(File);
		return auxDIBImageLoad(Filename); // Pointer auf Texturfile
	}
	return NULL; // Fehlerfall
}

///////////////////////////////////////////////////////////////////////
// Texturlader mit entsprechenden Texturdefinitionen
///////////////////////////////////////////////////////////////////////
void LoadGLTextures() {
AUX_RGBImageRec *TextureImage[11];
int i;

	TextureImage[0]=LoadBMP("01005.bmp");
	TextureImage[1]=LoadBMP("kohle.bmp");
	TextureImage[2]=LoadBMP("gras.bmp");
	TextureImage[3]=LoadBMP("back.bmp");
	TextureImage[4]=LoadBMP("schiene.bmp");
	
	for (i=0;i<=3;i++) {
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
		glGenTextures(1,&textures[i]);
		glBindTexture(GL_TEXTURE_2D,textures[i]);
		glTexImage2D(GL_TEXTURE_2D,0,3,TextureImage[i]->sizeX,TextureImage[i]->sizeY,0,GL_RGB,GL_UNSIGNED_BYTE,TextureImage[i]->data);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		free(TextureImage[i]->data);
		free(TextureImage[i]);
	}
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glGenTextures(1,&textures[4]);
	glBindTexture(GL_TEXTURE_2D,textures[4]);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,TextureImage[4]->sizeX,TextureImage[i]->sizeY,GL_RGB,GL_UNSIGNED_BYTE,TextureImage[4]->data);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	free(TextureImage[4]->data);
	free(TextureImage[4]);
}

///////////////////////////////////////////////////////////////////////
//	Anfang des OpenGL Programmes
///////////////////////////////////////////////////////////////////////
int main (int argc,char **argv) {
	// Fensterinitialisierung
	glutInit (&argc,argv);
	glutInitWindowSize (WIN_WIDTH,WIN_HEIGHT);
	glutInitWindowPosition (WIN_POS_X,WIN_POS_Y);
	glutInitDisplayMode (USED_MODUS);
	MainWin = glutCreateWindow (PROG_NAME);

	// OpenGL Initialisierungen

	LoadGLTextures();
	glClearColor(1,1,1,1);
	
	// Beleuchtungsmodell einstellen
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_NORMALIZE);
	glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	
	// globales ambientes Licht
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,global_ambient_light);
	
	// Punktlicht an dre Kameraposition
	glLightfv(GL_LIGHT7,GL_POSITION,camera_light_position);
	
	// globale Materialdefinition
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mat_color);
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,mat_color);
	glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,mat_emission);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,mat_shininess);
		
	glDepthFunc(GL_LEQUAL);
		
	// Callback Funktionen
	glutTimerFunc(1000,OnTimer,0);
	glutDisplayFunc (DisplayFunc);
	glutReshapeFunc (ReshapeFunc);
	glutMouseFunc (MouseFunc);
	glutMotionFunc (MouseMove);
	glutKeyboardFunc (KeyboardFunc);
	glutSpecialFunc (SpecialKeyboardFunc);
	glutIdleFunc (IdleFunc);

	// Initiale Mauseigenschaften
	globMouse.LastState = -1;
	globMouse.MoveX 	= 0;
	globMouse.MoveY 	= 0;
	globMouse.Radius 	= 15;

	// Die Hauptschleife
	glutMainLoop ();
	return 0;
}

///////////////////////////////////////////////////////////////////////
//	Display CALLBACK Funktion - wird immer aufgerufen, wenn der 
//	Inhalt des Viewports aktualisiert werden muss (z.B. nach
//	Aufruf der Funktion 'glutPostRedisplay ()').
///////////////////////////////////////////////////////////////////////
void DisplayFunc () {
double time1;
// dienen der Zeitmessung
int i;
// Laufvariable
GLfloat sun_position[]={sunx,suny,sunz,0};
GLfloat sun_diffuse[]={sun_intensity,sun_intensity,sun_intensity,1};
GLfloat sun_specular[]={sun_intensity,sun_intensity,sun_intensity,1};
GLfloat sun_ambient[]={0.3*sun_intensity,0.3*sun_intensity,0.3*sun_intensity,1};
// Parameter fuer die Sonne ergeben sich aus obigen Parametern

	// Buffer neu initialisieren
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();

	// Ansichtstransformationen
	double x,y,z, The,Phi;
	Phi = (double)globMouse.MoveX / (double)globMouse.ScreenX * M_PI * 2.0;
	The = (double)globMouse.MoveY / (double)globMouse.ScreenY * M_PI * 2.0;
	if (The>3*M_PI/2) {The = 0;globMouse.MoveY=0;}
	if (The>M_PI) {The = M_PI;globMouse.MoveY=0;}
	if (The>0.5*M_PI) {The = 0.5*M_PI;}
	z = globMouse.Radius * sin (Phi) * cos (The);
	x = globMouse.Radius * cos (Phi) * cos (The);
	y = globMouse.Radius * sin (The)+1;
	// Y muss >=0.5 sein
	y = MAX(y,0.5);
	int Oben = (The <= 0.5 * M_PI || The > 1.5 * M_PI) * 2 - 1;
	gluLookAt (x,y,z, 0,1,0, 0, Oben, 0);

	// Sonnenlicht
	glLightfv(GL_LIGHT0,GL_POSITION,sun_position);
	glLightfv(GL_LIGHT0,GL_SPECULAR,sun_specular);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,sun_diffuse);
	glLightfv(GL_LIGHT0,GL_AMBIENT,sun_ambient);

	// Headlights, 3 nach vorn, 3 nach hinten
	for (i=1;i<=3;i++) {
		glLightfv((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:GL_LIGHT3,GL_SPECULAR,(v>=0)?headlight_specular:headlight_red_specular);
		glLightfv((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:GL_LIGHT3,GL_DIFFUSE,(v>=0)?headlight_diffuse:headlight_red_diffuse);
		glLightfv((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:GL_LIGHT3,GL_SPOT_DIRECTION,headlight1_direction);
	}
	for (i=4;i<=6;i++) {
		glLightfv((i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_SPECULAR,(v<=0)?headlight_specular:headlight_red_specular);
		glLightfv((i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_DIFFUSE,(v<=0)?headlight_diffuse:headlight_red_diffuse);
		glLightfv((i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_SPOT_DIRECTION,headlight2_direction);
	}
	for (i=1;i<=6;i++) {
		glLightf((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:(i==3)?GL_LIGHT3:(i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_SPOT_CUTOFF,headlight_cutoff);
		glLightf((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:(i==3)?GL_LIGHT3:(i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_LINEAR_ATTENUATION,headlight_attenuation);
		glLightf((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:(i==3)?GL_LIGHT3:(i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_QUADRATIC_ATTENUATION,headlight_attenuation);
		glLightf((i==1)?GL_LIGHT1:(i==2)?GL_LIGHT2:(i==3)?GL_LIGHT3:(i==4)?GL_LIGHT4:(i==5)?GL_LIGHT5:GL_LIGHT6,GL_SPOT_CUTOFF,headlight_cutoff);
	}
	
	// Kameralicht
	glLightfv(GL_LIGHT7,GL_SPECULAR,camera_light_specular);
	glLightfv(GL_LIGHT7,GL_DIFFUSE,camera_light_diffuse);
	
	// Shading-Model
	(shading==1)?glShadeModel(GL_SMOOTH):glShadeModel(GL_FLAT);

	// Sonne an-/ausschalten
	(light0==1)?glEnable(GL_LIGHT0):glDisable(GL_LIGHT0);

	// Lokbeleuchtung an-/ausschalten
	if (light==1) {
		if (show[7]==1) {glEnable(GL_LIGHT1);glEnable(GL_LIGHT2);glEnable(GL_LIGHT3);}
		if (show[10]==1) {glEnable(GL_LIGHT4);glEnable(GL_LIGHT5);glEnable(GL_LIGHT6);}
	}
	else {
		glDisable(GL_LIGHT1);glDisable(GL_LIGHT2);glDisable(GL_LIGHT3);
		glDisable(GL_LIGHT4);glDisable(GL_LIGHT5);glDisable(GL_LIGHT6);
	}

	// Kameralicht an-/ausschalten
	(light1==1)?glEnable(GL_LIGHT7):glDisable(GL_LIGHT7);

	// Lighting an-/ausschalten
	(lighting==1)?glEnable(GL_LIGHTING):glDisable(GL_LIGHTING);

	// Wireframe oder gefuellte Polygone?
	glPolygonMode(GL_FRONT_AND_BACK,((wireframe==1)?GL_LINE:GL_FILL));

	// Z-Puffer aktivieren oder nicht
	(depth==1)?glEnable (GL_DEPTH_TEST):glDisable(GL_DEPTH_TEST);
	
	// Modellierungstransformationen
	time1=glutGet(GLUT_ELAPSED_TIME);  // erstmal Zeit nehmen
	Lok();	// das ganze Modell zusammensetzen
	glFlush ();				// Daten an Server (fuer die Darstellung) 
							// schicken
	glutSwapBuffers();		// Buffers wechseln
}

///////////////////////////////////////////////////////////////////////
//	Resize CALLBACK Funktion - wird aufgerufen, wenn sich die 
//	Fenstergroesse aendert
//		w,h: neue Breite und Hoehe des Fensters
///////////////////////////////////////////////////////////////////////
void ReshapeFunc (int w, int h) {
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	// Parallelprojektion oder Zentralprojektion?
	(ortho==1)?glOrtho(-6.0*(double)w/(double)h, 6*(double)w/(double)h, -6.0, 6.0, 1, 2000.0):gluPerspective (45, (double)w/(double)h, 1, 2000);
	glViewport (0, 0, w, h);
	glMatrixMode (GL_MODELVIEW);	// Sicherheit halber 
	glLoadIdentity ();				// Modelierungsmatrix einstellen
	globMouse.ScreenX = w;			
	globMouse.ScreenY = h;
}

///////////////////////////////////////////////////////////////////////
//	Maus Button CALLBACK Funktion
//		button - Welche Taste betaetigt bzw. losgelassen wurde
//		state  - Status der State (GL_DOWN, GL_UP)
//		x, y   - Fensterkoordinaten des Mauszeigers
///////////////////////////////////////////////////////////////////////
void MouseFunc (int button, int state, int x, int y) {
	globMouse.LastState = (state == GLUT_DOWN) ? button : -1;
	switch (button) {
		// Linke Taste - mit Rotation verknuepft
		case GLUT_LEFT_BUTTON:
			if (state == GLUT_DOWN) {
				globMouse.OldX = x;
				globMouse.OldY = y;
				}
			break;
		// Rechte Taste - mit Skalierung verknuepft
		case GLUT_RIGHT_BUTTON:
			if (state == GLUT_DOWN)
				globMouse.OldY = y;
			break;
		}
}

///////////////////////////////////////////////////////////////////////
//	Maus Movement CALLBACK Funktion, wenn das Mausbutton gedrueckt
//		wurde. Fuer reine Mausbewegung muss man GlutPassiveMotionFunc
//		einstellen.
///////////////////////////////////////////////////////////////////////
void MouseMove (int x, int y) {
	switch (globMouse.LastState) {
		case GLUT_LEFT_BUTTON:
			double The;
			The = (double)globMouse.MoveY / (double)globMouse.ScreenY * M_PI * 2.0;
			if (The>0.5*M_PI) The = 0.5*M_PI;
			if ((The!=0.5*M_PI)||(y-globMouse.OldY<0)) globMouse.MoveY += y - globMouse.OldY;
			if (globMouse.MoveY > globMouse.ScreenY)
				globMouse.MoveY -= globMouse.ScreenY;
			if (globMouse.MoveY < 0)
				globMouse.MoveY =+ globMouse.ScreenY;
			globMouse.MoveX += x - globMouse.OldX;
			if (globMouse.MoveX > globMouse.ScreenX)
				globMouse.MoveX -= globMouse.ScreenX;
			if (globMouse.MoveX < 0)
				globMouse.MoveX =+ globMouse.ScreenX;
			break;
		case GLUT_RIGHT_BUTTON:
			globMouse.Radius += y - globMouse.OldY;
			globMouse.Radius = MAX (globMouse.Radius,1);
			globMouse.Radius = MIN (globMouse.Radius,size*2.9);
			// herauszoomen begrenzt auf Umgebungswuerfel
			break;
		}
	globMouse.OldX = x;
	globMouse.OldY = y;
	glutPostRedisplay ();
}

///////////////////////////////////////////////////////////////////////
//	Tastatur CALLBACK Funktion
//		key: Wert der Taste die gedrueckt wurde
//		x,y: Position des Mauskursors auf dem Viewport
///////////////////////////////////////////////////////////////////////
void KeyboardFunc (unsigned char key, int x, int y) {
	switch (key) {
		// Escape==Programmabbruch
		case 27:
			exit (0);
		// Initialisierung==Zurueck auf Startposition
		case '0':
			globMouse.MoveX = 0;
			globMouse.MoveY = 0;
			glutPostRedisplay ();
			break;
		// Autodetail an/aus
		case 'a':autodetail=1-autodetail;break;
		// Modell links-rechts verschieben
		case 'r':shiftx=1-shiftx;break;
		// Modell vorwaerts/rueckwaerts verschieben
		case 't':if (shiftz<2.9*(size-6)) shiftz+=1;break;
		case 'T':if (shiftz>-2.9*(size-4)) shiftz-=1;break;
		// Z-Puffer an/aus
		case 'z':depth=1-depth;
			(depth==1)?glEnable(GL_DEPTH_TEST):glDisable(GL_DEPTH_TEST);break;
		// Wireframe an/aus
		case 'w':wireframe=1-wireframe;
			glPolygonMode(GL_FRONT_AND_BACK,((wireframe==1)?GL_LINE:GL_FILL));break;
		// Lighting an/aus
		case 'l':lighting=1-lighting;
			(lighting==1)?glEnable(GL_LIGHTING):glDisable(GL_LIGHTING);break;
		// Smooth/Flat-Shading umschalten
		case 's':shading=1-shading;
			(shading==1)?glShadeModel(GL_SMOOTH):glShadeModel(GL_FLAT);break;
		// Headlights (Lampen der Lok) anschalten
		case 'h':light=1-light;break;
		// Detailgrad fuer Zylinder und Disks
		case ',':slices+=1;break;
		case '.':if (slices>mindetail) slices-=1;break;
		// Tempo der Lok beeinflussen
		case '+':if (v<120) v+=0.5;break;
		case '-':if (v>-50) v-=0.5;break;
		// Fuellstand des Tenders
		case 'K':(fill>0)?fill-=0.01:fill=fill;break;
		case 'k':(fill<1)?fill+=0.01:fill=fill;break;
		// Groesse des Umgebungswuerfels
		case 'g':if (size>7) size-=1;
				 shiftz=0;globMouse.Radius = MIN (globMouse.Radius,size*2.9);
				 break;
		case 'G':size+=1;break;
		// Clipping-Plane an/aus
		case 'p':clip=1-clip;break;
		// Parallel- oder Zentralprojektion
		case 'o':ortho=1-ortho;
			ReshapeFunc(glutGet(GLUT_WINDOW_WIDTH),glutGet(GLUT_WINDOW_HEIGHT));break;
		// Sonne an/aus
		case '1':(light0==1)?glEnable(GL_LIGHT0):glDisable(GL_LIGHT0);
			light0=1-light0;break;
		// Kameralicht an/aus
		case '2':light1=1-light1;
			(light1==1)?glEnable(GL_LIGHT7):glDisable(GL_LIGHT7);break;
		// Headlight-Cutoff veraendern
		case 'c':if (headlight_cutoff>1) headlight_cutoff-=1;break;
		case 'C':if (headlight_cutoff<90) headlight_cutoff+=1;break;
		// Headlight-Attenuation veraendern
		case 'U':if (headlight_attenuation>0) headlight_attenuation-=0.001;break;
		case 'u':if (headlight_attenuation<1) headlight_attenuation+=0.001;break;
		// Sonnenposition veraendern
		case 'b':if (sunx>-1) sunx-=0.1;break;
		case 'B':if (sunx<1) sunx+=0.1;break;
		case 'n':if (suny>0) suny-=0.1;break;
		case 'N':if (suny<1) suny+=0.1;break;
		case 'm':if (sunz>-1) sunz-=0.1;break;
		case 'M':if (sunz<1) sunz+=0.1;break;
		// Sonnenintensitaet veraendern
		case 'i':if (sun_intensity>0) sun_intensity-=0.01;break;
		case 'I':if (sun_intensity<1) sun_intensity+=0.01;break;
		case '*':scale=scale*2;break;
		case '/':if (scale>0.04) scale=scale/2;break;
		}
	glutPostRedisplay();
}

//
//	CALLBACK Funktion fuer spezielle Tasten (Kursortasten, F-Tasten usw.)
//		key - Code der Taste (siehe glut Spezifikation)
//
void SpecialKeyboardFunc (int key, int x, int y) {
	switch (key) {
		// mit den F-Tasten die einzelnen Teile der Lok anzeigen
		case GLUT_KEY_F1:show[0]=1-show[0];break;
		case GLUT_KEY_F2:show[1]=1-show[1];break;
		case GLUT_KEY_F3:show[2]=1-show[2];break;
		case GLUT_KEY_F4:show[3]=1-show[3];break;
		case GLUT_KEY_F5:show[4]=1-show[4];break;
		case GLUT_KEY_F6:show[5]=1-show[5];break;
		case GLUT_KEY_F7:show[6]=1-show[6];break;
		case GLUT_KEY_F8:show[7]=1-show[7];break;
		case GLUT_KEY_F9:show[8]=1-show[8];break;
		case GLUT_KEY_F10:show[9]=1-show[9];break;
		case GLUT_KEY_F11:show[10]=1-show[10];break;
		}
	glutPostRedisplay ();
}


//
//	CALLBACK Funktion fuer Idle: wird aufgerufen, wenn die CPU nichts
//	anderes zu tun hat: oft fuer Animation verwendet 
//
void IdleFunc () {
double d=2.0; // Durchmesser Treibraeder
	time1=time2;
	time2=glutGet(GLUT_ELAPSED_TIME); // nochmal Zeit nehmen
	timediff=(time2-time1)/1000;	  // Differenz bestimmen fuer fps
	angle+=(100*v*timediff)/(d*M_PI);
	if (angle>360) angle-=360*(int)(angle/360);
	glutPostRedisplay();
	++fps;
}

//
//	CALLBACK Funktion fuer Timer: zhlt fps und setzt detailstufe
//
void OnTimer (int value) {
char temp[80];
	if (autodetail==1) {
		if ((fps<minfps/2)&&(slices>10)) slices/=2;
		if (fps<minfps*3/4) scale=1;
		if ((fps<minfps)&&(slices>10)) slices-=1;
		if ((fps<minfps)&&(size>7)) size-=1;
		if ((fps>maxfps*2)&&(scale<4)) scale*=2;
		if (fps>1.5*maxfps) slices*=2;
		if (fps>maxfps) slices+=1;
		if (fps>maxfps) size+=1;
	}
	sprintf(temp,"%s [%d fps] [%d slices] [%f km/h]",PROG_NAME,fps,slices,v);
	glutSetWindowTitle(temp);
	glutTimerFunc(1000,OnTimer,0);
	fps=0;
}

///////////////////////////////////////////////////////////////////////
//
//	Modellierungsfunktionen
//
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
// Standardquadriken: Zylinder wahlweise mit Deckel oben und/oder unten
// sowie Disk, beides mit variabler Aufloesung
///////////////////////////////////////////////////////////////////////
void Disk(GLUquadric* quad, double r) {
double detail;
	detail=r*slices;
	gluDisk(quad,0,r,(detail<mindetail)?mindetail:detail,(r<1)?1:r);
}

void Zylinder(GLUquadric* quad, double r, double l,bool oben, bool unten) {
double detail;
	detail=r*slices;
	gluCylinder(quad,r,r,l,(detail<mindetail)?mindetail:detail,(l<1)?2:l);
	if (unten) {
		Disk(quad,r);
	}
	if (oben) { 
		glTranslatef(0,0,l);
		Disk(quad,r);
		glTranslatef(0,0,-l);
	}
}

///////////////////////////////////////////////////////////////////////
//
// Kessel der Lok inkl Schornstein, Dom und Sandkasten
//
///////////////////////////////////////////////////////////////////////

void Kessel(GLUquadric* quad,double posy, double posz,double l1,double l2,double d,double bx,double by, double d1, double pos1, double h1,double d2, double pos2, double h2, double d3, double pos3, double h3, double d4, double pos4, double h4, double d5, double pos5, double h5) {
// posz:	Abstand hinteres Lokende (==Koordursprung) zu hinterem Kesselende
// l1,l2:	Lnge des Kessels gesamt, Lnge des Stehkessels
// d:		Durchmesser Kessel
// d1..d3	Durchmesser Schornstein, 1. Dom, 2. Dom
// bx,by:	Position des unteren Endes des Aschkastens (gerechnet von Kesselmittelachse und hinterem Kesselende)
// pos1..3: Position des Schornsteins, 1. Dom, 2. Dom, gemessen vom hinteren Kesselende
// h1..3:	Hhe des Schornsteins, 1. Dom, 2. Dom (gemessen ab Kesseloberkante)
// alle Angaben in Metern, hinteres Kesselende ist da, wo das Fhrerhaus beginnt
	
	glPushMatrix();
		glColor3ubv(schwarz);
		// Langkessel
		glTranslatef(0,posy,posz);
		Zylinder(quad,d/2,l1,FALSE, FALSE);

		// ringfoermige Disk und Sphere als Kesseltuer
		glPushMatrix();
			glTranslatef(0,0,l1);
			gluDisk(quad,0.85*d/2,d/2,(d2/2*slices<mindetail)?mindetail:slices,(d2/2<1)?1:d2/2);
	
			// Sphere wird zusaetzlich noch gestaucht
			glScalef(1,1,0.1);
			glutSolidSphere(0.85*d/2,(d/2*slices<mindetail)?mindetail:d/2*slices,(d/2*slices<mindetail)?mindetail:d/2*slices);
		glPopMatrix();

		// Stehkessel als Wuerfel
		glPushMatrix();
			glTranslatef(0,-d/4,l2/2);
			glScalef(d,d/2,l2-0.01);
			glutSolidCube(1);
		glPopMatrix();

		// Aschkasten
		glBegin(GL_TRIANGLES);
			glNormal3f(1,0,0);glVertex3f(d/2,-d/2,l2);glVertex3f(d/2,bx,by);glVertex3f(d/2,-d/2,0);
			glNormal3f(-1,0,0);glVertex3f(-d/2,-d/2,l2);glVertex3f(-d/2,bx,by);glVertex3f(-d/2,-d/2,0);
		glEnd();
		glBegin(GL_QUADS);
			glNormal3f(0,-d*(l2-by),-d*(bx+d/2));glVertex3f(d/2,-d/2,l2);glVertex3f(-d/2,-d/2,l2);glVertex3f(-d/2,bx,by);glVertex3f(d/2,bx,by);
			glNormal3f(0,-d*by,d*(bx+d/2));glVertex3f(-d/2,bx,by);glVertex3f(d/2,bx,by);glVertex3f(d/2,-d/2,0);glVertex3f(-d/2,-d/2,0);
		glEnd();

		// Schornstein
		glPushMatrix();
			glTranslatef(0,d/2+(d/2)*(sin(acos(d1/d))-1),pos1);
			glRotatef(-90,1,0,0); // Schornstein zeigt ja nach oben
			Zylinder(quad,d1/2,h1-(d/2)*(sin(acos(d1/d))-1),FALSE,FALSE);
	
			// eine nichtreflektierende Scheibe, damit man im Schornstein nicht
			// den Kessel spiegeln sieht
			glTranslatef(0,0,-(d/2)*(sin(acos(d1/d))-1)+0.01);
			glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,rauch_specular);
			Disk(quad,d1/2);
			glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mat_color);
		glPopMatrix();
		
		// Kesselblock zu den Zylindern
		glPushMatrix();
			glTranslatef(0,-1,pos1);
			glScalef(2.0,1.8,0.5);
			glutSolidCube(1);
		glPopMatrix();
	
		// 1. Dom
		glPushMatrix();
			glTranslatef(0,d/2+(d/2)*(sin(acos(d2/d))-1),pos2);
			glRotatef(-90,1,0,0);
			Zylinder(quad,d2/2,h2-(d/2)*(sin(acos(d2/d))-1),TRUE, FALSE);
		glPopMatrix();

		// 2. Dom (sofern vorhanden) (ist bei der BR01 der Fall)
		if (d3>0) {
			glPushMatrix();
				glTranslatef(0,d/2+(d/2)*(sin(acos(d3/d))-1),pos3);
				glRotatef(-90,1,0,0);
				Zylinder(quad,d3/2,h3-(d/2)*(sin(acos(d3/d))-1),TRUE, FALSE);
			glPopMatrix();
		}

		// 1. Sandkasten
		glPushMatrix();
			glTranslatef(0,d/2+(d/2)*(sin(acos(d4/d))-1),pos4);
			glRotatef(-90,1,0,0);
			glScalef(1,0.5,1);
			Zylinder(quad,d4/2,h4-(d/2)*(sin(acos(d4/d))-1),TRUE, FALSE);
		glPopMatrix();
		
	
		// 2. Sandkasten (sofern vorhanden) (bei BR01 _nicht_ der Fall)
		if (d5>0) {
			glPushMatrix();
				glTranslatef(0,d/2+(d/2)*(sin(acos(d5/d))-1),pos5);
				glRotatef(-90,1,0,0);
				glScalef(1,0.5,1);
				Zylinder(quad,d5/2,h5-(d/2)*(sin(acos(d5/d))-1),TRUE, FALSE);
			glPopMatrix();
		}


		// Nummer vorn an der Kesseltr
		glColor3ubv(weiss);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D,textures[0]);
		glBegin(GL_QUADS);
			glNormal3f(0,0,1);
			glTexCoord2f(0,1);glVertex3f(-0.3,-0.1,l1+0.1);
			glTexCoord2f(1,1);glVertex3f(0.3,-0.1,l1+0.1);
			glTexCoord2f(1,0);glVertex3f(0.3,-0.25,l1+0.1);
			glTexCoord2f(0,0);glVertex3f(-0.3,-0.25,l1+0.1);
		glEnd();
		glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////////
//
// ab hier Funktionen, die zur Darstellung des Fuehrerhauses dienen
//
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
// Seitenwand als QUAD_STRIP, Sichtschutz als Zylinder
///////////////////////////////////////////////////////////////////////
void Seitenwand_Sichtschutz(GLUquadric* quad,double posy,double l1,double h1,double h2,double b1,double b2,double x1,double y1,double x2,double y2,double x3,double y3,double x4,double y4) {
// der schraege Teil der Seitenwand mit "Loechern" fuer die Fenster
// und die beiden Schutzschirme gegen einfallendes Sonnenlicht
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Seitenwaende
// posy : Grundhoehe der Unterkante des Fuehrerhauses ueber Schienenhoehe
// l1   : Laenge des Fuehrerhauses
// h1   : Hoehe des unteren senkrechten Teils der Seitenwand
// h2   : Hoehe des oberen schraeg stehenden Teils der Seitenwand
// b1   : Breite des Fuehrerhauses unten
// b2   : Breite des Fuehrerhauses oben
// die x,y-Koord der Fenster sind der Abstand von (0,h1)
// x1,y1: Position der unteren linken Ecke des hinteren Fensters
// x2,y2: Position der oberen rechten Ecke des hinteren Fensters
// x3,y3: Position der unteren linken Ecke des vorderen Fensters
// x4,y4: Position der oberen rechten Ecke des vorderen Fensters
	
	for (side=-1;side<=1;side=side+2) {
		// erstmal den schraegen Teil der Seitenwand als QUAD_STRIP
		glBegin(GL_QUAD_STRIP);
			glNormal3f(-side*(h1-h2),-(b2-b1),0);
			glVertex3f(side*b1/2,posy+h1,l1/2);glVertex3f(side*(b1/2)-side*((y1*(b1-b2)/2)/h2),posy+h1+y1,x2);
			glVertex3f(side*b1/2,posy+h1,0);glVertex3f(side*(b1/2)-side*((y1*(b1-b2)/2)/h2),posy+h1+y1,x1);
			glVertex3f(side*b2/2,posy+h1+h2,0);glVertex3f(side*(b1/2)-side*((y2*(b1-b2)/2)/h2),posy+h1+y2,x1);
			glVertex3f(side*b2/2,posy+h1+h2,l1/2);glVertex3f(side*(b1/2)-side*((y2*(b1-b2)/2)/h2),posy+h1+y2,x2);
			glVertex3f(side*(b1/2)-side*((y4*(b1-b2)/2)/h2),posy+h1+y4,x3);glVertex3f(side*(b1/2)-side*((y1*(b1-b2)/2)/h2),posy+h1+y1,x2);
			glVertex3f(side*(b1/2)-side*((y3*(b1-b2)/2)/h2),posy+h1+y3,x3);glVertex3f(side*b1/2,posy+h1,l1/2);
			glVertex3f(side*(b1/2)-side*((y3*(b1-b2)/2)/h2),posy+h1+y3,x4);glVertex3f(side*b1/2,posy+h1,l1);
			glVertex3f(side*(b1/2)-side*((y4*(b1-b2)/2)/h2),posy+h1+y4,x4);glVertex3f(side*b2/2,posy+h1+h2,l1);
			glVertex3f(side*(b1/2)-side*((y4*(b1-b2)/2)/h2),posy+h1+y4,x3);glVertex3f(side*b2/2,posy+h1+h2,l1/2);
		glEnd();
		
		// und hier nun den Sichtschutz anbauen
		glPushMatrix();
			glTranslatef(side*b2/2,posy+h1+h2/2,l1);
			glRotatef(side*atan((b1-b2)/(2*h2))*180/M_PI,0,0,1);
			glScalef(0.25,1,1);
			gluDisk(quad,0.35,0.52,(0.35*slices<mindetail)?mindetail:0.35*slices,1);
			Zylinder(quad,0.35,0.4,FALSE,FALSE);
		glPopMatrix();
	}

}

///////////////////////////////////////////////////////////////////////
// Fuehrerhaus aus QUADS aufgebaut
///////////////////////////////////////////////////////////////////////
void Fuhrerhaus(GLUquadric* quad,double posy,double l1,double l2,double h1,double h2,double h3,double b1,double b2,double x1,double y1,double x2,double y2,double x3,double y3,double x4,double y4) {
GLdouble plane[4]={0,1,0,0};
// Clipping-Plane fuer den Zylinder, der das Dach bildet
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Seitenwaende
// posy : Grundhoehe der Unterkante des Fuehrerhauses ueber Schienenhoehe
// l1   : Laenge des Fuehrerhauses
// l2   : Laenge des Daches
// h1   : Hoehe des unteren senkrechten Teils der Seitenwand
// h2   : Hoehe des oberen schraeg stehenden Teils der Seitenwand
// b1   : Breite des Fuehrerhauses unten
// b2   : Breite des Fuehrerhauses oben
// die x,y-Koord der Fenster sind der Abstand von (0,h1)
// x1,y1: Position der unteren linken Ecke des hinteren Fensters
// x2,y2: Position der oberen rechten Ecke des hinteren Fensters
// x3,y3: Position der unteren linken Ecke des vorderen Fensters
// x4,y4: Position der oberen rechten Ecke des vorderen Fensters
	glPushMatrix();
		glColor3ubv(schwarz);
		
		// zuerst den Boden und die unteren Waende als QUADS zeichnen
		glBegin(GL_QUADS);
			// Boden
			glNormal3f(0,1,0);glVertex3f(b1/2,posy,0);glVertex3f(-b1/2,posy,0);glVertex3f(-b1/2,posy,l1);glVertex3f(b1/2,posy,l1);
			// jetzt Seitenwand unten links
			glNormal3f(1,0,0);glVertex3f(b1/2,posy,0);glVertex3f(b1/2,posy+h1,0);glVertex3f(b1/2,posy+h1,l1);glVertex3f(b1/2,posy,l1);
			// und Seitenwand unten rechts
			glNormal3f(1,0,0);glVertex3f(-b1/2,posy,0);glVertex3f(-b1/2,posy+h1,0);glVertex3f(-b1/2,posy+h1,l1);glVertex3f(-b1/2,posy,l1);	
			// und die Vorderwand unten
			glNormal3f(0,0,1);glVertex3f(b1/2,posy,l1);glVertex3f(-b1/2,posy,l1);glVertex3f(-b1/2,posy+h1,l1);glVertex3f(b1/2,posy+h1,l1);
		glEnd();
		
		// dann Vorderwand oben mit Schlitzen fuer Sicht nach vorn als QUAD_STRIP
		glBegin(GL_QUAD_STRIP);
			glNormal3f(0,0,1);
			glVertex3f(0,posy+h1,l1);
			glVertex3f(b2/2-0.0875+0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2-0.35,l1);
			glVertex3f(b1/2,posy+h1,l1);
			glVertex3f(b2/2+0.0875+0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2-0.35,l1);
			glVertex3f(b2/2,posy+h1+h2,l1);
			glVertex3f(b2/2+0.0875-0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2+0.35,l1);
			glVertex3f(0,posy+h1+h2,l1);
			glVertex3f(b2/2-0.0875-0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2+0.35,l1);
			glVertex3f(-b2/2+0.0875+0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2+0.35,l1);
			glVertex3f(b2/2-0.0875+0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2-0.35,l1);
			glVertex3f(-b2/2+0.0875-0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2-0.35,l1);
			glVertex3f(0,posy+h1,l1);
			glVertex3f(-b2/2-0.0875-0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2-0.35,l1);
			glVertex3f(-b1/2,posy+h1,l1);
			glVertex3f(-b2/2-0.0875+0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2+0.35,l1);
			glVertex3f(-b2/2,posy+h1+h2,l1);
			glVertex3f(-b2/2+0.0875+0.35*sin(atan((b1-b2)/(2*h2))),posy+h1+h2/2+0.35,l1);
			glVertex3f(0,posy+h1+h2,l1);
		glEnd();

		// jetzt die oberen Seitenwaende und die Sichtschutzblenden
		Seitenwand_Sichtschutz(quad,posy,l1,h1,h2,b1,b2,x1,y1,x2,y2,x3,y3,x4,y4);

		// und Dach obendrauf
		glTranslatef(0,posy+h1+h2,l1-l2);
		glScalef(1,h3/(b2/2),1);
		glClipPlane(GL_CLIP_PLANE0,plane);
		glEnable(GL_CLIP_PLANE0);
		Zylinder(quad,b2/2,l2,TRUE,FALSE);
		glDisable(GL_CLIP_PLANE0);
	glPopMatrix();

	// zum Schluss Nummernschilder an den Seite
	glColor3ubv(weiss);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,textures[0]);
	for (side=-1;side<=1;side=side+2) {
	glBegin(GL_QUADS);
		glNormal3f(1,0,0);
		glTexCoord2f((side<0)?1:0,1);glVertex3f(side*(b1/2+0.01),posy+h1/2+0.075,l1/2+0.3);
		glTexCoord2f((side<0)?1:0,0);glVertex3f(side*(b1/2+0.01),posy+h1/2-0.075,l1/2+0.3);
		glTexCoord2f((side<0)?0:1,0);glVertex3f(side*(b1/2+0.01),posy+h1/2-0.075,l1/2-0.3);
		glTexCoord2f((side<0)?0:1,1);glVertex3f(side*(b1/2+0.01),posy+h1/2+0.075,l1/2-0.3);
	glEnd();
	}
	glDisable(GL_TEXTURE_2D);
	glColor3ubv(schwarz);
}

///////////////////////////////////////////////////////////////////////
//
// Rahmen unter der Lok
//
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
// Funktion zum Zeichnen der Puffer
///////////////////////////////////////////////////////////////////////
void Puffer(GLUquadric* quad) {
int side;
	for (side=-1;side<=1;side=side+2) {
			glPushMatrix();
				glColor3ubv(rot);
				glTranslatef(side*0.788,0,0);
				Zylinder(quad,0.1,0.65,FALSE,FALSE);
				glColor3ubv(schwarz);
				glTranslatef(0,0,0.65);
				Disk(quad,0.212);
			glPopMatrix();
		}
}

void Rahmen(GLUquadric* quad,double breite,double l1,double l2,double l3,double l4, double l5, double l6, double h1, double h2, double h3,double h4, double x1, double x2, double x3, double x4, double x5) {
// breite:  Breite des Rahmens
// l1..4:	Lnge der einzelnen Rahmenteile 
// (1: Schleppachse und Fhrerstand, 2: unterm Aschkasten, 3: Treibachsen, 4: Laufachse)
// l5		mittlere Position des Drehgestells (Position der Drehachse)
// l6		Lnge Drehgestell (Radstand + berhang)
// h1..4:	Hhen der einzelnen Rahmenteile
// x1..4:	absolute Hhe des Rahmenteils (Unterkante bis Schiene)
// x5		vertikale mittlere Position des Drehgestells (entspricht Radius der Laufachse(n))
	glPushMatrix();
		glColor3ubv(rot);
		
		// zuerst das Drehgestell fuer die Laufachse(n), sofern vorhanden
		if (x5!=0) {
			glPushMatrix();
				glColor3ubv(rot);
				glTranslatef(0,x5,l5);
				glScalef(breite,0.3,l6);
				glutSolidCube(1);
				glScalef(1/breite,1/0.3,1/l6);
				glRotatef(-90,1,0,0);
				Zylinder(quad,0.4,0.5,FALSE,FALSE);
			glPopMatrix();
		}

		// der eigentliche Rahmen unterm Fuehrerstand
		glPushMatrix();
			glTranslatef(0,x1+h1/2,l1/2);
			glScalef(breite,h1,l1);
			glutSolidCube(1);
		glPopMatrix();
			
		// unterm Aschkasten
		glPushMatrix();
			glTranslatef(0,x2+h2/2,l1+l2/2);
			glScalef(breite,h2,l2);
			glutSolidCube(1);
		glPopMatrix();
			
		// an den Treibachsen
		glPushMatrix();
			glTranslatef(0,x3+h3/2,l1+l2+l3/2);
			glScalef(breite,h3,l3);
			glutSolidCube(1);
		glPopMatrix();
		
		// und ganz nach vorne
		glPushMatrix();
			glTranslatef(0,x4+h4/2,l1+l2+l3+l4/2);
			glScalef(breite,h4,l4);
			glutSolidCube(1);
		glPopMatrix();
		
		// und davor dann das Frontblech, wo die Puffer rankommen
		glPushMatrix();
			glTranslatef(0,1.025,l1+l2+l3+l4+0.1);
			glScalef(2.0,0.35,0.4);
			glutSolidCube(1);
		glPopMatrix();
			
		// Puffer vorn rechts und links
		glTranslatef(0,1.025,l1+l2+l3+l4+0.1);
		Puffer(quad);
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////////
//
// ab hier Funktionen zum Zeichnen der Raeder und Achsen
//
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
// Zeichnen der Speichen eines Rades als Zylinder
///////////////////////////////////////////////////////////////////////
void Speichen(GLUquadric* quad, int anzahl, double l) {
int a;
// a: Laufvariable
// anzahl: Anzahl der Speichen
// l: Durchmesser des Rades, damit doppelte Laenge der Speichen

	for (a=0;a<anzahl;a++) {
		glPushMatrix();
			glRotatef((360/anzahl)*a,0,0,1);
			glRotatef(90,0,1,0);
			glScalef(1,0.5,1);
			Zylinder(quad,0.07,(l/2)-0.1,FALSE,FALSE);
		glPopMatrix();
	}
}

///////////////////////////////////////////////////////////////////////
// zeichnen der kompletten Achse inkl. der Raeder auf beiden Seiten
// realisiert als Zeichnung zweier Halbachsen
///////////////////////////////////////////////////////////////////////
void Halbachse(GLUquadric* quad,double d1,double d2,int anzahl,double gewicht) {
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Halbachsen
GLdouble plane[4]={0,-1,0,-gewicht};
// plane : Clipping-Plane fuer die Gegengewichte
// d1 Durchmesser des Rades
// d2 Durchmesser der Nabe
// anzahl: Anzahl speichen
// gewicht: Groesse des Gegengewichtes, wenn Gegengewicht vorhanden, so wird 
// ein Bolzen fuer die Aufnahme der Kuppelstange mitgezeichnet
	// Schleife, in der beide Halbachsen gezeichnet werden
	for (side=-1;side<=1;side=side+2) {
		glPushMatrix();
			glRotatef(-90,0,1,0);
			if (side>0) glRotatef(180,1,0,0);
			// Rotation um um Bewegung zu simulieren
			glRotatef(side*angle/(d1/2),0,0,1);

			// die Achse mitsamt Nabe zeichnen
			glPushMatrix();
				glColor3ubv(schwarz);
				Zylinder(quad,d2/2,0.8175,FALSE, FALSE);
				glTranslatef(0,0,0.8175);
				glColor3ubv(rot);
				Disk(quad,d2/2);
			glPopMatrix();

			// das eigentliche Rad zeichnen
			glPushMatrix();
				glTranslatef(0,0,0.6675);

				// zuerst der Spurkranz
				glColor3ubv(grau);
				gluCylinder(quad,(d1/2)+0.05,d1/2,0.05,(d1/2*slices<mindetail)?mindetail:d1/2*slices,(d1/2<1)?1:d1/2);				
				// dann die Laufflaeche
				glTranslatef(0,0,0.05);
				Zylinder(quad,d1/2,0.1,FALSE,FALSE);
				glColor3ubv(rot);
				// Radreifen rechts nnen
				// die Orientierung wird geaendert, um eine korrektere
				// Beleuchtung zu erhalten
				glTranslatef(0,0,-0.05);
				gluQuadricOrientation(quad,GLU_INSIDE);
				gluDisk(quad,(d1/2)-0.1,(d1/2)+0.05,(d1/2*slices<mindetail)?mindetail:d1/2*slices,(d1/2<1)?1:d1/2);

				// die Innenflaeche des Radreifens wird auch so gezeichnet
				Zylinder(quad,(d1/2)-0.1,0.15,FALSE,FALSE);
				gluQuadricOrientation(quad,GLU_OUTSIDE);

				// die Aussenflaeche des Radreifens
				glTranslatef(0,0,0.15);
				gluDisk(quad,(d1/2)-0.1,d1/2,((d1/2)*slices<mindetail)?mindetail:(d1/2)*slices,((d1/2)<1)?1:d1/2);

				// wenn ein Gegengewicht vorhanden ist, so wird dieses 
				// zusammen mit einem Bolzen fuer die Kuppelstange gezeichnet
				if (gewicht!=0.0) {
					glColor3ubv(schwarz);
					glPushMatrix();
						glTranslatef(0,d2/2,0);
						// der Kupplungsbolzen
						Zylinder(quad,0.075,0.17,TRUE, FALSE);
					glPopMatrix();
					glColor3ubv(rot);

					// dann das eigentliche Gegengewicht
					glClipPlane(GL_CLIP_PLANE0,plane);
					glEnable(GL_CLIP_PLANE0);
					Disk(quad,d1/2);
					glTranslatef(0,0,-0.15);

					// der innere Teil des Gegengewichtes wird nochmals
					// anderes orientiert aus Beleichtungsgruenden
					glRotatef(180,0,1,0);
					Disk(quad,d1/2);
						
					glDisable(GL_CLIP_PLANE0);
				}

				// und zum Schluss die Speichen
				glTranslatef(0,0,-0.075);
				Speichen(quad,anzahl,d1);
			glPopMatrix();
		glPopMatrix();
	}
}

///////////////////////////////////////////////////////////////////////
// Zusammenstellen eines beliebigen Radsatzes
///////////////////////////////////////////////////////////////////////
void Radsatz(GLUquadric* quad,double y[4],int s[4],double n[4],double g[4],double z[6],int t[6]) {
int a;
// a: Laufvariable
// 4 Radtypen: Schleppachse, Kuppelachse, Treibachse, Laufachse
// y: Feld fuer Raddurchmesser
// s: Feld fuer Speichenzahlen
// n: Feld fuer Nabendurchmesser
// g: Feld fuer Groesse der Gegengewichte
// z: Feld von Radposition (fuer 6 Raeder)
// t: Anordnung der Raeder nach Typ
	glPushMatrix();
		for (a=0;a<=5;a++) {
			if (t[a]!=0) {
				glTranslatef(0,y[t[a]-1]/2,z[a]);
				Halbachse(quad,y[t[a]-1],n[t[a]-1],s[t[a]-1],g[t[a]-1]);
				glTranslatef(0,-y[t[a]-1]/2,0);
			}
		}
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////////
// Windleitbleche, Trittbleche, Frontabdeckungen, Hauptluftbehaelter
///////////////////////////////////////////////////////////////////////
void Bleche(GLUquadric* quad,double posx,double posy,double posz,double l,double b,double h2, double l2,double b2,double x1,double x2,double wagl1,double wagh1,double wagl2,double wagh2,double luftx,double lufty,double luftz,double luftr,double luftl,double lx1,double ly1,double lz1,double lx2,double ly2) {
GLfloat light_emission[]={light,light,light,0.0};
GLfloat light_red_emission[]={light,0,0,0.0};
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Haelften
// posx, posy,posz: hinteres Ende Trittblech
// l,b: Lnge Breite der waagerechten Bleche entlang des Kessels, b2 Breite vordere Stirnseiten auen
// h2: Hhendifferenz Horizontalblech-vorderes Ende
// l2: waagerechte Differenz von Horizontalblech zu vorderem Ende
// x1,x2: horizontale Positionen des mittleren Frontblendestcks, x1 gemessen von posz, x2 als Differenz zu x1
// wagl1: Entfernung hinteres Ende zu Knick unten Leitblech, wagl2: Entfernung Ende schrges Trittblech-vorderes Ende Windleitblech
// wagh1: Hhe des Windleitblechs gemessen vom Trittblech aus
// wagh2: Hhe des angeschrgten Teils des Windleitblechs
// luftx,y,z: Position Hauptluftbehlter
// luftr: Radius Hauptluftbehlter
// luftl: Lnge Hauptluftbehlter
// Bei anderen Baureihen liegen Hauptluftbehaelter evtl nich so schoen symmetrisch!!
// lx1,ly1,lz1: Position der Stirnlampen unten (an x spiegeln)
// lx2,ly2: Position der mittleren Stirnlampe
	for (side=-1;side<=1;side=side+2) {
	glBegin(GL_QUADS);
		glColor3ubv(schwarz);
		
		// zuerst Trittblech seitlich am Kessel
		glNormal3f(0,1,0);
		glVertex3f(side*posx,posy,posz);glVertex3f(side*posx,posy,posz+l);
		glVertex3f(side*(posx-b),posy,posz+l);glVertex3f(side*(posx-b),posy,posz);
		
		// Anschluss vorn schraeg (Frontblech aussen)
		glNormal3f(0,l2,h2);
		glVertex3f(side*posx,posy,posz+l);glVertex3f(side*posx,posy-h2,posz+l+l2);
		glVertex3f(side*(posx-b2),posy-h2,posz+l+l2);glVertex3f(side*(posx-b2),posy,posz+l);
		
		// Frontblech in der Mitte
		glNormal3f(0,x2,h2);
		glVertex3f(side*(posx-b2),posy,posz+x1);glVertex3f(side*(posx-b2),posy-h2,posz+x1+x2);
		glVertex3f(-side*(posx-b2),posy-h2,posz+x1+x2);glVertex3f(-side*(posx-b2),posy,posz+x1);
		
		// Verbindungsteil zwischen mittlerem und aeusserem Frontblech
		glNormal3f(1,0,0);
		glVertex3f(side*(posx-b2),posy,posz+x1);glVertex3f(side*(posx-b2),posy,posz+l);
		glVertex3f(side*(posx-b2),posy-h2,posz+l+l2);glVertex3f(side*(posx-b2),posy-h2,posz+x1+x2);
		
		// obere Trittstufe auf der mittleren Blende
		glNormal3f(0,1,0);
		glVertex3f(side*(posx-b2),posy-h2/3,posz+x1+x2/3);glVertex3f(-side*(posx-b2),posy-h2/3,posz+x1+x2/3);
		glVertex3f(-side*(posx-b2),posy-h2/3,posz+x1+x2/3+0.3);glVertex3f(side*(posx-b2),posy-h2/3,posz+x1+x2/3+0.3);
		
		// untere Trittstufe auf der mittleren Blende
		glNormal3f(0,1,0);
		glVertex3f(side*(posx-b2),posy-2*h2/3,posz+x1+2*x2/3);glVertex3f(-side*(posx-b2),posy-2*h2/3,posz+x1+2*x2/3);
		glVertex3f(-side*(posx-b2),posy-2*h2/3,posz+x1+2*x2/3+0.3);glVertex3f(side*(posx-b2),posy-2*h2/3,posz+x1+2*x2/3+0.3);
		
		// roter Zierstreifen am Trittblech waagerecht
		glColor3ubv(rot);glNormal3f(side*1,0,0);
		glVertex3f(side*posx,posy,posz);glVertex3f(side*posx,posy,posz+l);
		glVertex3f(side*posx,posy-0.05,posz+l);glVertex3f(side*posx,posy-0.05,posz);
		
		// roter Zierstreifen an der Frontblende
		glVertex3f(side*posx,posy,posz+l);glVertex3f(side*posx,posy-h2,posz+l+l2);
		glVertex3f(side*posx,posy-h2-0.05,posz+l+l2);glVertex3f(side*posx,posy-0.05,posz+l);
		
		// Verbindungsblech fuer die Luftbehaelter
		glNormal3f(0,1,0);
		glVertex3f(0,lufty,luftz);glVertex3f(0,lufty,luftz+luftl);
		glVertex3f(side*luftx,lufty,luftz+luftl);glVertex3f(side*luftx,lufty,luftz);
	glEnd();
	glColor3ubv(schwarz);
	
	// nun die Luftleitbleche (Bauart Wagner, Witte muessten neu modelliert werden)
	glBegin(GL_QUAD_STRIP);
		glNormal3f(1,0,0);
		glVertex3f(side*posx,posy-h2,posz+l+l2);glVertex3f(side*posx,posy-h2,posz+l+l2+wagl2);
		glVertex3f(side*posx,posy,posz+l);glVertex3f(side*posx,posy,posz+l+l2+wagl2);
		glVertex3f(side*posx,posy,posz+l-wagl1);glVertex3f(side*posx,posy+wagh1/2-wagh2,posz+l+l2+wagl2);
		glVertex3f(side*posx,posy+wagh1-wagh2,posz+l-wagl1);glVertex3f(side*posx,posy+wagh1-wagh2,posz+l+l2+wagl2);
		// oberer schraeg stehender Teil
		glNormal3f(wagh2,wagh2/12,0);
		glVertex3f(side*(posx-wagh2/12),posy+wagh1-wagh2/2,posz+l-wagl1);glVertex3f(side*(posx-wagh2/12),posy+wagh1-wagh2/2,posz+l+l2+wagl2-wagh2/6);
		glNormal3f(wagh2,wagh2/4,0);
		glVertex3f(side*(posx-wagh2/4),posy+wagh1,posz+l-wagl1);glVertex3f(side*(posx-wagh2/4),posy+wagh1,posz+l+l2+wagl2-wagh2/2);
	glEnd();

	// fuer eine ordentliche Beleuchtung der Innenseite der Windleitbleche gibts eine 
	// weitere Schicht mit negiertem Normalenvektor
	glBegin(GL_QUAD_STRIP);
		glNormal3f(-1,0,0);
		glVertex3f(side*(posx-0.01),posy-h2,posz+l+l2);glVertex3f(side*(posx-0.01),posy-h2,posz+l+l2+wagl2);
		glVertex3f(side*(posx-0.01),posy,posz+l);glVertex3f(side*(posx-0.01),posy,posz+l+l2+wagl2);
		glVertex3f(side*(posx-0.01),posy,posz+l-wagl1);glVertex3f(side*(posx-0.01),posy+wagh1/2-wagh2,posz+l+l2+wagl2);
		glVertex3f(side*(posx-0.01),posy+wagh1-wagh2,posz+l-wagl1);glVertex3f(side*(posx-0.01),posy+wagh1-wagh2,posz+l+l2+wagl2);
		// oberer schraeg stehender Teil
		glNormal3f(-wagh2,-wagh2/12,0);
		glVertex3f(side*(posx-wagh2/12-0.01),posy+wagh1-wagh2/2,posz+l-wagl1);glVertex3f(side*(posx-wagh2/12-0.01),posy+wagh1-wagh2/2,posz+l+l2+wagl2-wagh2/6);
		glNormal3f(-wagh2,-wagh2/4,0);
		glVertex3f(side*(posx-wagh2/4-0.01),posy+wagh1,posz+l-wagl1);glVertex3f(side*(posx-wagh2/4-0.01),posy+wagh1,posz+l+l2+wagl2-wagh2/2);
	glEnd();
	glColor3ubv(rot);
	
	// jetzt Schluss der Hauptluftbehaelter
	glPushMatrix();
		glTranslatef(side*luftx,lufty,luftz);
		gluCylinder(quad,luftr,luftr,luftl,(luftr*slices<mindetail)?mindetail:luftr*slices,(luftl<1)?2:luftl);
		glScalef(1,1,0.5);
		glutSolidSphere(luftr,(luftr*slices<mindetail)?mindetail:luftr*slices,(luftr*slices<mindetail)?mindetail:luftr*slices);
		glTranslatef(0,0,2*luftl);
		glutSolidSphere(luftr,(luftr*slices<mindetail)?mindetail:luftr*slices,(luftr*slices<mindetail)?mindetail:luftr*slices);
	glPopMatrix();
	}

	// zum Schluss die Beleuchtung
	glPushMatrix();
		for (side=-1;side<=1;side=side+2) {
			// erst die beiden Lampen unten
			glColor3ubv(schwarz);
			glPushMatrix();
				glTranslatef(side*lz1,ly1,lx1);
				// Lampengehaeuse
				Zylinder(quad,0.12,0.2,FALSE,TRUE);
				glTranslatef(0,0,0.2);
				glColor3ub(255,255,255);
				glMaterialfv(GL_FRONT,GL_EMISSION,(v>=0)?light_emission:light_red_emission);
				// "Glas", sendet Licht aus
				Disk(quad,0.12);
				glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,mat_emission);
				glLightfv((side<0)?GL_LIGHT1:GL_LIGHT2,GL_POSITION,headlight_position);
			glPopMatrix();
		}
		glColor3ubv(schwarz);
		// dann Stirnlampe vorn oben mitte
		glTranslatef(0,ly2,lx2);
		// Lampengehaeuse
		Zylinder(quad,0.12,0.2,FALSE,TRUE);
		glTranslatef(0,0,0.2);
		glColor3ub(255,255,255);
		glMaterialfv(GL_FRONT,GL_EMISSION,(v>=0)?light_emission:light_red_emission);
		// "Glas", sendet Licht aus
		Disk(quad,0.12);
		glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,mat_emission);
		glLightfv(GL_LIGHT3,GL_POSITION,headlight_position);
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////////
//
// ab hier Funktionen zum Zeichnen des Tenders
//
///////////////////////////////////////////////////////////////////////
void TenderRahmen(GLUquadric* quad,double posx,double posy,double x1,double l1,double x2,double l2,double l3,double breite,double y,double dl1,double posx1,double dl2,double posx2) {
int side,i;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Haelften
// i: Laufvariable
// posy Hhe der Unterkante des Wasserkastens
// posx: Abstand Lok Tender
// l1,h1:Mae Wasserkasten
// l2,h2:Mae Kohlebunker
// l3,h3: Mae des dritten Kastens ganz hinten
// x1,x2: horizontale Position der Ksten gemessen von Tenderbeginn vorne
// breite: Breite des Drehgestellrahmens
// dl1,dl2: Laenge des ersten bzw. 2. Drehgestells
// y: Hoehe des Drehgestells ueber der Schiene
// posx1,posx2: Position der Drehgestelle vom Koordursprung aus

	// Rahmen am der Seite
	glTranslatef(0,0,-posx);
	for (side=-1;side<=1;side=side+2) {
	glColor3ubv(rot);
	glBegin(GL_QUADS);
		glNormal3f(side,0,0);
		glVertex3f(side,1.025+0.173,0);glVertex3f(side,1.025+0.173,-x1-l1-l3);
		glVertex3f(side,1.025-0.173,-x1-l1-l3);glVertex3f(side,1.025-0.173,0);
		// Pufferbohle
		glNormal3f(0,0,-1);
		glVertex3f(1,1.198,-x1-l1-l3);glVertex3f(1,0.852,-x1-l1-l3);
		glVertex3f(-1,0.852,-x1-l1-l3);glVertex3f(-1,1.198,-x1-l1-l3);
		// Rahmenblock vorn
		glNormal3f(0,0,1);
		glVertex3f(1,1.198,0);glVertex3f(1,0.852,0);
		glVertex3f(-1,0.852,0);glVertex3f(-1,1.198,0);
		glNormal3f(0,1,0);
		glVertex3f(1,1.198,0);glVertex3f(1,1.198,-x1);
		glVertex3f(-1,1.198,-x1);glVertex3f(-1,1.198,0);
	glEnd();
	}
	// Verbindung zwischen Lok und Tender (Kupplung sozusagen)
	glPushMatrix();
		glTranslatef(0,posy-0.3,posx/2);
		glRotatef(-90,1,0,0);
		Zylinder(quad,0.15,0.3,TRUE,TRUE);
	glPopMatrix();
	
	// Rahmenblock
	glPushMatrix();
		glTranslatef(0,posy-0.173,-(x1+l1+l3)/2);
		glScalef(1,0.346,x1+l1+l3);
		glutSolidCube(1);
	glPopMatrix();
	
	// Puffer
	glPushMatrix();
		glTranslatef(0,1.025,-x1-l1-l3);
		glRotatef(180,0,1,0);
		Puffer(quad);
	glPopMatrix();
	glColor3ubv(rot);
	glTranslatef(0,0,posx);
	// Drehgestelle 
	for (side=1;side<=2;side++) {
		glPushMatrix();
			glTranslatef(0,y,(side==1)?-posx1:-posx2);
			glScalef(breite,0.3,(side==1)?dl1+0.5:dl2+0.5);
			glutSolidCube(1);
			glScalef(1/breite,1/0.3,1/((side==1)?dl1+0.5:dl2+0.5));
			glRotatef(-90,1,0,0);
			Zylinder(quad,0.4,0.5,FALSE,FALSE);
		glPopMatrix();
		for (i=-1;i<=1;i=i+2) {
			// Drehgestellrahmen aussen
			glBegin(GL_QUADS);
				glNormal3f(0,1,0);
				glVertex3f(i*0.85,y,(side==1)?-posx1+0.3:-posx2+0.3);
				glVertex3f(0,y,(side==1)?-posx1+0.3:-posx2+0.3);
				glVertex3f(0,y,(side==1)?-posx1-0.3:-posx2-0.3);
				glVertex3f(i*0.85,y,(side==1)?-posx1-0.3:-posx2-0.3);
				glNormal3f(i,0,0);
				glVertex3f(i*0.85,y+0.2,(side==1)?-posx1+dl1/2+0.25:-posx2+dl2/2+0.25);
				glVertex3f(i*0.85,y-0.2,(side==1)?-posx1+dl1/2+0.25:-posx2+dl2/2+0.25);
				glVertex3f(i*0.85,y-0.2,(side==1)?-posx1-dl1/2-0.25:-posx2-dl2/2-0.25);
				glVertex3f(i*0.85,y+0.2,(side==1)?-posx1-dl1/2-0.25:-posx2-dl2/2-0.25);
			glEnd();
		}
	}
}

///////////////////////////////////////////////////////////////////////
// der eigentliche Tender mit Kohle- und Wasserbunker sowie Lampen
///////////////////////////////////////////////////////////////////////
void Tender(GLUquadric* quad,double posx,double posy,double x1,double h1,double l1,double x2,double h2,double l2,double h3,double l3,double ly1,double lz1,double ly2) {
GLfloat light_emission[]={light,light,light,0.0};
GLfloat light_red_emission[]={light,0.0,0.0,0.0};
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Haelften
// posy Hhe der Unterkante des Wasserkastens
// posx: Abstand Lok Tender
// l1,h1:Mae Wasserkasten
// l2,h2:Mae Kohlebunker
// l3,h3: Mae des dritten Kastens ganz hinten
// x1,x2: horizontale Position der Ksten gemessen von Tenderbeginn vorne
// ly1,lz1: Position der Stirnlampen unten (an x spiegeln)
// ly2: Position der mittleren Stirnlampe
	glPushMatrix();
		glColor3ubv(schwarz);
		glTranslatef(0,0,-posx);

		// Wasserkasten
		glPushMatrix();
			glTranslatef(0,posy+h1/2,-x1-l1/2);
			glScalef(3.0,h1,l1);
			glutSolidCube(1);
		glPopMatrix();

		// Kohlebunker
		glBegin(GL_QUADS);
			glNormal3f(0,0,1);
			glVertex3f(0.95,posy+h1,-x2);glVertex3f(0.95,posy+h1+h2,-x2);
			glVertex3f(-0.95,posy+h1+h2,-x2);glVertex3f(-0.95,posy+h1,-x2);
			glNormal3f(-1,0,0);
			glVertex3f(-0.95,posy+h1,-x2);glVertex3f(-0.95,posy+h1+h2,-x2);
			glVertex3f(-0.95,posy+h1+h2,-x2-l2);glVertex3f(-0.95,posy+h1,-x2-l2);
			glNormal3f(0,0,1);
			glVertex3f(0.95,posy+h1,-x2-l2);glVertex3f(0.95,posy+h1+h2,-x2-l2);
			glVertex3f(-0.95,posy+h1+h2,-x2-l2);glVertex3f(-0.95,posy+h1,-x2-l2);
			glNormal3f(1,0,0);
			glVertex3f(0.95,posy+h1,-x2);glVertex3f(0.95,posy+h1+h2,-x2);
			glVertex3f(0.95,posy+h1+h2,-x2-l2);glVertex3f(0.95,posy+h1,-x2-l2);
			//Trittblech fr Heizer
			glNormal3f(0,1,0);
			glVertex3f(1.5,1.636,0);glVertex3f(1.5,1.636,-x1);
			glVertex3f(-1.5,1.636,-x1);glVertex3f(-1.5,1.636,0);
		glEnd();
	
		// Kohletextur obendrauf auf den Tender
		glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D,textures[1]);
			glColor3ub(255,255,255);
			glBegin(GL_QUADS);
				glNormal3f(0,1,0);
				glTexCoord2f(0,0);glVertex3f(0.95,posy+h1+h2-fill,-x2-l2);
				glTexCoord2f(1,0);glVertex3f(-0.95,posy+h1+h2-fill,-x2-l2);
				glTexCoord2f(1,1);glVertex3f(-0.95,posy+h1+h2-fill,-x2);
				glTexCoord2f(0,1);glVertex3f(0.95,posy+h1+h2-fill,-x2);
			glEnd();
		glDisable(GL_TEXTURE_2D);

		glColor3ubv(schwarz);
		// Kasten ganz hinten
		glPushMatrix();
			glTranslatef(0,1.198+h3/2,-x1-l1-l3/2);
			glScalef(2.0,h3,l3);
			glutSolidCube(1);
		glPopMatrix();

		for (side=-1;side<=1;side=side+2) {
		glBegin(GL_QUADS);
			// Tueren
			glNormal3f(1,0,0);
			glVertex3f(side*1.5,1.7,0);glVertex3f(side*1.5,1.7,-x1);
			glVertex3f(side*1.5,1.7+h1/2,-x1);glVertex3f(side*1.5,1.7+h1/2,0);
		glEnd();
		}
		// zum Schluss die Beleuchtung
		glPushMatrix();
			glRotatef(180,0,1,0);
			for (side=-1;side<=1;side=side+2) {
				// erst die beiden Lampen unten
				glColor3ubv(schwarz);
				glPushMatrix();
					glTranslatef(side*lz1,ly1,x1+l1+l3);
					// Lampengehaeuse
					Zylinder(quad,0.12,0.2,FALSE,TRUE);
					glTranslatef(0,0,0.2);
					glColor3ub(255,255,255);
					glMaterialfv(GL_FRONT,GL_EMISSION,(v<=0)?light_emission:light_red_emission);
					// "Glas", sendet Licht aus
					Disk(quad,0.12);
					glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,mat_emission);
					glLightfv((side<0)?GL_LIGHT4:GL_LIGHT5,GL_POSITION,headlight_position);
				glPopMatrix();
			}
			glColor3ubv(schwarz);
			// dann Stirnlampe vorn oben mitte
			glTranslatef(0,ly2,x1+l1);
			// Lampengehaeuse
			Zylinder(quad,0.12,0.2,FALSE,TRUE);
			glTranslatef(0,0,0.2);
			glColor3ub(255,255,255);
			glMaterialfv(GL_FRONT,GL_EMISSION,(v<=0)?light_emission:light_red_emission);
			// "Glas", sendet Licht aus
			Disk(quad,0.12);
			glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,mat_emission);
			glLightfv(GL_LIGHT6,GL_POSITION,headlight_position);
		glPopMatrix();
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////////
// Tenderradsatz, alle Raeder gleich
///////////////////////////////////////////////////////////////////////
void TenderRadsatz(GLUquadric* quad,double y,int s,double posx,double x1,double x2,double x3,double x4,double x5) {
// posx	horizontaler Offset
// y1: Durchmesser des Rades, x1: Abstand zum vorhergehenden bzw zum vorderen Ende des Tenders
// s: Anzahl der Speichen
	glPushMatrix();
				
		// 1. Achse
		glTranslatef(0,y/2,-posx-x1);
		Halbachse(quad,y,0.3,s,0);

		// 2. Achse
		glTranslatef(0,0,-x2);
		Halbachse(quad,y,0.3,s,0);

		// 3. Achse
		glTranslatef(0,0,-x3);
		Halbachse(quad,y,0.3,s,0);

		// 4. Achse, wenn vorhanden
		if (x4!=0) {
		glTranslatef(0,0,-x4);
		Halbachse(quad,y,0.3,s,0);
		}
	
		// 5. Achse, wenn vorhanden (hier nicht der Fall)
		if (x5!=0) {
		glTranslatef(0,0,-x5);
		Halbachse(quad,y,0.3,s,0);
		}
	glPopMatrix();
}

///////////////////////////////////////////////////////////////////////
// das Herzstueck der Animation, viele Rotationen und trigonometrische
// Funktionen, Fachbegriffe sind in der Dokumentation erlaeutert
///////////////////////////////////////////////////////////////////////
void Triebwerk(GLUquadric* quad,double d,double posx,double posy,double posz,double l1,double x1,double x2,double x3,double x4,double x5,double x6,double x7,double x8,double x9, double x10,double x11,double x12) {
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Haelften
int offset;
// offset: Winkelversatz, Triebwerke arbeiten um 180 Grad versetzt
// d: Durchmesser Treibraeder
// posx, posy: Achse des Rades, an dem die Treibstange befestigt ist
// x1,x2    Lnge Kuppelstange vorn/hinten
// x3	Lnge Treibstange
// x4	Lnge Kolbenstange
// x5	Lnge Gegenkurbel
// x6	Lnge Schwingenstange
// x7	Abstand Treibachse-Fixpunkt Schwinge horizontal
// x8	Lnge Schwinge
// x9	Lnge Lenkerstange
// x10  Lnge Voreilhebel
// x11	Lnge Steuerstange
// x12	Lnge Schieberstange
	for (side=-1;side<=1;side=side+2) {
	(side<0)?offset=180:offset=0;
	glColor3ubv(schwarz);
	glPushMatrix();
		
		// langer Bolzen an der Treibachse
		glTranslatef(side*posz,posy,posx);	
		glRotatef(angle+offset,1,0,0);
		glTranslatef(0,-l1,0);
		glRotatef(side*90,0,1,0);
		Zylinder(quad,0.075,0.40,TRUE, FALSE);
		glPushMatrix();
			
		// verschiedene Transformationen zur Gegenkurbel
			glColor3ubv(rot);
			glTranslatef(0,0,0.35);
			glRotatef(side*90,0,1,0);
			glRotatef(-114,1,0,0);
			glColor3ubv(schwarz);
			// Gegenkurbel
			gluCylinder(quad,0.04,0.02,x5,(0.04*slices<mindetail)?mindetail:0.04*slices,(x5<1)?1:x5);
			
			// Bolzen oben an der Gegenkurbel
			glTranslatef(0,0,x5);
			glRotatef(-side*90,0,1,0);
			Zylinder(quad,0.03,0.1,TRUE, FALSE);
			glTranslatef(0,0,0.07);
			glColor3ubv(rot);
		
			// Schwingenstange
			glRotatef(-side*90,0,1,0);
			glRotatef(-(114+angle+offset)-acos((1000*x7-((x6-x7)*1000*cos((-90+angle+offset)*M_PI/180)))/(1001*x6))*180/M_PI,1,0,0);
			Zylinder(quad,0.025,x6,TRUE,TRUE);
			
			// Bolzen zwischen Schwingenstange und Schwinge
			glTranslatef(0,0,x6);
			glRotatef(side*90,0,1,0);
			glTranslatef(0,0,-0.03);
			Zylinder(quad,0.025,0.06,TRUE,TRUE);

			// Schwinge
			glTranslatef(0,0,0.03);
			glRotatef(-side*90,0,1,0);
			glRotatef(-19.5*cos((angle+offset)*M_PI/180)-90+acos((1000*x7-(x6-x7)*1000*cos((-90+angle+offset)*M_PI/180))/(1001*x6))*180/M_PI,1,0,0);
			Zylinder(quad,0.03,x8,TRUE,TRUE);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0,0,0.1);
			
			// Kuppelstange nach hinten
			glRotatef(-side*(angle+offset),0,0,1);
			glRotatef(side*90,0,1,0);
			glColor3ubv(rot);
			Zylinder(quad,0.06,x2,FALSE,FALSE);
		glPopMatrix();

		glPushMatrix();
			glTranslatef(0,0,0.1);
			
			// Kuppelstange nach vorn
			glRotatef(-side*(angle+offset),0,0,1);
			glRotatef(-side*90,0,1,0);
			glColor3ubv(rot);
			Zylinder(quad,0.06,x1,FALSE,FALSE);
		glPopMatrix();
		glPushMatrix();
			
			// Treibstange
			glTranslatef(0,0,0.24);
			glRotatef(-side*(angle+offset)-side*asin(cos((angle+offset)*M_PI/180)*l1/x3)*180/M_PI,0,0,1);
			glRotatef(-side*90,0,1,0);
			glColor3ubv(rot);
			Zylinder(quad,0.07,x3,FALSE,FALSE);
			
			// Bolzen zwischen Treibstange und Kreuzkopf
			glColor3ubv(schwarz);
			glTranslatef(0,0,x3);
			glRotatef(side*90,0,1,0);
			glTranslatef(0,0,-0.1);
			Zylinder(quad,0.075,0.2,TRUE,TRUE);

			// Transformationen zum Kreuzkopf
			glTranslatef(0,0,0.1);
			glRotatef(-side*90,0,1,0);
			glRotatef(side*asin(side*cos((angle+offset)*M_PI/180)*l1/x3)*180/M_PI,1,0,0);
			glPushMatrix();
				glColor3ubv(grau);
				glPushMatrix();
					// Kreuzkopf
					glTranslatef(side*0.075,-0.075,0.1);
					glScalef(0.15,0.15,0.2);
					glutSolidCube(1);
				glPopMatrix();
				
				// Lenkerstange
				glTranslatef(side*0.15,-0.15,0);
				glRotatef(-asin((-38*cos(2*(angle+offset)*M_PI/180)-19)/(1000*x9))*180/M_PI,1,0,0);
				Zylinder(quad,0.03,x9,FALSE,FALSE);
				
				// Voreilhebel
				glTranslatef(0,0,x9);
				glRotatef(asin(l1*1000*sin((angle+offset)*M_PI/180)/(x10*1000))*180/M_PI-90+asin((-38*cos(2*(angle+offset)*M_PI/180)-19)/(1000*x9))*180/M_PI,1,0,0);
				Zylinder(quad,0.03,x10,FALSE,FALSE);
				
				// Schieberstange
				glTranslatef(0,0,x10*0.9);
				glRotatef(90-asin(l1*1000*sin((angle+offset)*M_PI/180)/(x10*1000))*180/M_PI,1,0,0);
				Zylinder(quad,0.02,x12,FALSE,FALSE);
				
				// Steuerstange
				glRotatef(-90+asin(l1*1000*sin((angle+offset)*M_PI/180)/(x10*1000))*180/M_PI,1,0,0);
				glTranslatef(0,0,x10*0.1);
				glColor3ubv(rot);
				glRotatef(-90-asin(l1*1000*sin((angle+offset)*M_PI/180)/(x10*1000))*180/M_PI,1,0,0);
				Zylinder(quad,0.04,x11,FALSE,FALSE);
			glPopMatrix();
			
			// Kolbenstange
			glColor3ubv(grau);
			Zylinder(quad,0.06,x4,FALSE,FALSE);
			glColor3ubv(schwarz);
		glPopMatrix();
	glPopMatrix();
	}
}

///////////////////////////////////////////////////////////////////////
// 2 Zylinder, reicht fuer viele Loktypen
///////////////////////////////////////////////////////////////////////
void Zylinderblock(GLUquadric* quad,double r1,double r2,double h,double b,double posx,double posy,double posz,double l1,double l2) {
int side,i;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Haelften
// i : Laufvariable
glColor3ubv(schwarz);
for (side=-1;side<=1;side=side+2) {
	glPushMatrix();
		glTranslatef(side*posz,posy,posx);
		glScalef(side,1,1);
	
		// Vorder- und Hinterwand des Zylinderblocks
		for (i=-1;i<=1;i=i+2) {
		glBegin(GL_QUAD_STRIP);
			glNormal3f(0,0,1);
			glVertex3f(0,0,i*l1/2);glVertex3f(0,h+r1,i*l1/2);
			glVertex3f(r2,0,i*l1/2);glVertex3f(r2+b,h+r1,i*l1/2);
			glVertex3f(r2+r2*cos(atan(b/h)),-r2*sin(atan(b/h)),i*l1/2);
			glVertex3f(r2+b+r1*cos(atan(b/h)),h-r1*sin(atan(b/h)),i*l1/2);
		glEnd();
		}
	
		glBegin(GL_QUADS);
			// obere Wand
			glNormal3f(0,1,0);
			glVertex3f(0,h+r1,l1/2);glVertex3f(0,h+r1,-l1/2);
			glVertex3f(r2+b,h+r1,-l1/2);glVertex3f(r2+b,h+r1,l1/2);
			// innere Wand
			glNormal3f(1,0,0);
			glVertex3f(0,0,l1/2);glVertex3f(0,0,-l1/2);
			glVertex3f(0,h+r1,-l1/2);glVertex3f(0,h+r1,l1/2);
			// uere Wand
			glNormal3f(h,-b,0);
			glVertex3f(r2+r2*cos(atan(b/h)),-r2*sin(atan(b/h)),l1/2);
			glVertex3f(r2+r2*cos(atan(b/h)),-r2*sin(atan(b/h)),-l1/2);
			glVertex3f(r2+b+r1*cos(atan(b/h)),h-r1*sin(atan(b/h)),-l1/2);
			glVertex3f(r2+b+r1*cos(atan(b/h)),h-r1*sin(atan(b/h)),l1/2);
		glEnd();

		// Hauptzylinder
		glTranslatef(r2,0,-0.05-l1/2);
		Zylinder(quad,r2,l1+0.1,TRUE,TRUE);
			
		// Steuerzylinder
		glTranslatef(b,h,l1/2-l2/2);
		Zylinder(quad,r1,l2+0.1,TRUE,TRUE);
	glPopMatrix();
	}
}

///////////////////////////////////////////////////////////////////////
// "Umgebungswuerfel" mit Gras- und Schienentextur
///////////////////////////////////////////////////////////////////////
void Umgebung() {
int d=2; // Durchmesser Treibrder
int a,b;
	// Zuerst das Gras aufm Boden
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,textures[2]);
	glColor3ub(255,255,255);
	for (a=0;a<=5.8*size*scale;a++) 
	for (b=0;b<=5.8*size*scale;b++)
	{
	glBegin(GL_QUADS);
		glNormal3f(0,1,0);
		glTexCoord2f(0,angle*scale/45);glVertex3f(-2.9*size+a/scale,-0.01,-2.9*size+b/scale);
		glTexCoord2f(1,angle*scale/45);glVertex3f(-2.9*size+(1+a)/scale,-0.01,-2.9*size+b/scale);
		glTexCoord2f(1,1+angle*scale/45);glVertex3f(-2.9*size+(1+a)/scale,-0.01,-2.9*size+(b+1)/scale);
		glTexCoord2f(0,1+angle*scale/45);glVertex3f(-2.9*size+a/scale,-0.01,-2.9*size+(b+1)/scale);
	glEnd();
	}
	
	// dann die "Wnde" 
	glBindTexture(GL_TEXTURE_2D,textures[3]);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
	if(glGetError()) {
       glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
       glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
     } else
       glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);glVertex3f(2.9*size,-0.1,-2.9*size);
		glTexCoord2f(1,0);glVertex3f(2.9*size,-0.1,2.9*size);
		glTexCoord2f(1,1);glVertex3f(2.9*size,2.9*size,2.9*size);
		glTexCoord2f(0,1);glVertex3f(2.9*size,2.9*size,-2.9*size);
	glEnd();
	glBindTexture(GL_TEXTURE_2D,textures[3]);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);glVertex3f(-2.9*size,-0.1,2.9*size);
		glTexCoord2f(1,0);glVertex3f(-2.9*size,-0.1,-2.9*size);
		glTexCoord2f(1,1);glVertex3f(-2.9*size,2.9*size,-2.9*size);
		glTexCoord2f(0,1);glVertex3f(-2.9*size,2.9*size,2.9*size);
	glEnd();
	glBindTexture(GL_TEXTURE_2D,textures[3]);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);glVertex3f(-2.9*size,-0.1,2.9*size);
		glTexCoord2f(1,0);glVertex3f(2.9*size,-0.1,2.9*size);
		glTexCoord2f(1,1);glVertex3f(2.9*size,2.9*size,2.9*size);
		glTexCoord2f(0,1);glVertex3f(-2.9*size,2.9*size,2.9*size);
	glEnd();
	glBindTexture(GL_TEXTURE_2D,textures[3]);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);glVertex3f(2.9*size,-0.1,-2.9*size);
		glTexCoord2f(1,0);glVertex3f(-2.9*size,-0.1,-2.9*size);
		glTexCoord2f(1,1);glVertex3f(-2.9*size,2.9*size,-2.9*size);
		glTexCoord2f(0,1);glVertex3f(2.9*size,2.9*size,-2.9*size);
	glEnd();
		
	// zum Schlu 2 Gleise
	
	glBindTexture(GL_TEXTURE_2D,textures[4]);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	for (a=0;a<6*size;a++) {
	glBegin(GL_QUADS);
		glNormal3f(0,1,0);
		glTexCoord2f(0,angle/45);glVertex3f(1.7,0,-2.9*size+a);
		glTexCoord2f(1,angle/45);glVertex3f(-1.7,0,-2.9*size+a);
		glTexCoord2f(1,1+angle/45);glVertex3f(-1.7,0,2.9*(-size)+1+a);
		glTexCoord2f(0,1+angle/45);glVertex3f(1.7,0,2.9*(-size)+1+a);
	glEnd();
	}
	for (a=0;a<6*size;a++) {
	glBegin(GL_QUADS);
		glNormal3f(0,1,0);
		glTexCoord2f(0,angle/45);glVertex3f(3.4+1.7,0,-2.9*size+a);
		glTexCoord2f(1,angle/45);glVertex3f(3.4-1.7,0,-2.9*size+a);
		glTexCoord2f(1,1+angle/45);glVertex3f(3.4-1.7,0,2.9*(-size)+1+a);
		glTexCoord2f(0,1+angle/45);glVertex3f(3.4+1.7,0,2.9*(-size)+1+a);
	glEnd();
	}
	glDisable(GL_TEXTURE_2D);
}

///////////////////////////////////////////////////////////////////////
// als Schmankerl noch ein paar Glasscheiben eingesetzt
///////////////////////////////////////////////////////////////////////
void Seitenfenster(int side,double posy,double h1,double h2,double b1,double b2,double x3,double y3,double x4,double y4) {
	glBegin(GL_QUADS);
		// Seitenfenster zeichnen
		glNormal3f(-side*(h1-h2),-(b2-b1),0);
		glVertex3f(side*(b1/2-((y3*(b1-b2)/2)/h2)),posy+h1+y3,x3);
		glVertex3f(side*(b1/2-((y3*(b1-b2)/2)/h2)),posy+h1+y3,x4);
		glVertex3f(side*(b1/2-((y4*(b1-b2)/2)/h2)),posy+h1+y4,x4);
		glVertex3f(side*(b1/2-((y4*(b1-b2)/2)/h2)),posy+h1+y4,x3);
	glEnd();
}
void Schutzfenster(GLUquadric* quad,int side,double posy,double h1,double h2,double b1,double b2,double l1) {
	glPushMatrix();
		glTranslatef(-side*b2/2,posy+h1+h2/2,l1);
		glRotatef(-side*atan((b1-b2)/(2*h2))*180/M_PI,0,0,1);
		glScalef(0.25,1,1);
		Disk(quad,0.35);
	glPopMatrix();
}

void Fenster(GLUquadric* quad,double posy,double l1,double l2,double h1,double h2,double h3,double b1,double b2,double x1,double y1,double x2,double y2,double x3,double y3,double x4,double y4) {
int side;
// side : entweder 1 oder -1, fuer symmetrisches Zeichnen beider Haelften
// Parameter identisch Fuhrerhaus
double x,The,Phi;
// Variablen aus der Ansichtstransformation, werden gebraucht, um momentanen
// Standort zu bestimmen und richtige Reihenfolge (Tiefe) beim Zeichnen 
// einzuhalten (wird aber bei Sichtschutz nicht beachtet, weil hier keine 
// Anomalien auftreten koennen
Phi = (double)globMouse.MoveX / (double)globMouse.ScreenX * M_PI * 2.0;
The = (double)globMouse.MoveY / (double)globMouse.ScreenY * M_PI * 2.0;
x = globMouse.Radius * cos (Phi) * cos (The);

	// Blending aktivieren
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glColor4fv(window);

	// je nach Standort mit bestimmter Seite anfangen
	(x<0)?side=-1:side=1;
	Seitenfenster(side,posy,h1,h2,b1,b2,x3,y3,x4,y4);
	Schutzfenster(quad,side,posy,h1,h2,b1,b2,l1);
	Schutzfenster(quad,-side,posy,h1,h2,b1,b2,l1);
	Seitenfenster(-side,posy,h1,h2,b1,b2,x3,y3,x4,y4);
	glDisable(GL_BLEND);
}

///////////////////////////////////////////////////////////////////////
// die Grundfunktion, von hier werden alle anderen Funktionen mit 
// entsprechenden Parametern aufgerufen
///////////////////////////////////////////////////////////////////////
void Lok () {
GLdouble clipplane[4]={1,0,0,0};
// zusaetzliche Clipping-Plane
double y[4]={1.25,2.0,2.0,0.85};
// y0: Durchmesser Schleppachse, y1: Kuppelachse, y3: Treibachse, y4: Laufachse
double n[4]={0.3,0.66,0.66,0.3};
// n0: Durchmesser Schleppachsnabe, n1: Kuppelachsnabe, n3: Treibachsnabe, n4: Laufachsnabe
int s[4]={11,21,21,9};
// s0: Speichenzahl Schleppachse, s1: Kuppelachse, s3: Treibachse, s4: Laufachse
double g[4]={0.0,0.75,0.6,0.0};
// g0: Gegengewicht Schleppachse, g1: Kuppelachse, g3: Treibachse, g4: Laufachse
double z[6]={0.895,3.8,2.3,2.3,1.8,2.2};
// z0: Abstaende der Achsen von der vorhergehenden, erste Achse gemessen von hinterem Lokende
int t[6]={1,2,3,2,4,4};
// t0-6: Typ der jeweiligen Achse von hinten nach vorn (aus 2C1 mach 1C2), zusaetzlich 
// unterschieden in Treib- und Kuppelachsen

	// neue Quadrik erstellen
	GLUquadric* quad;
	quad=gluNewQuadric();

	glPushMatrix();
		// Aufruf der einzelnen Zeichenfunktionen mitsamt Parametern
		if (show[0]==1) Umgebung();
		glTranslatef(shiftx*3.4,0,shiftz);
		if (clip==1) {  
			glClipPlane(GL_CLIP_PLANE1,clipplane);
			glEnable(GL_CLIP_PLANE1); }

// void Radsatz(GLUquadric* quad,double y[4],int s[4],double n[4],double g[4],double z[6],int t[6]) {		
		if (show[1]==1) Radsatz(quad,y,s,n,g,z,t);

// void Rahmen(GLUquadric* quad,double breite,double l1,double l2,double l3,double l4, double l5, double l6, double h1, double h2, double h3,double h4, double x1, double x2, double x3, double x4, double x5) {
		if (show[2]==1) Rahmen(quad,1.30,1.771,2.55,6.130,3.85,12.195,2.6,1.0,0.486,0.862,0.35,0.6,0.728,0.768,0.850,0.5);

// void Kessel(GLUquadric* quad,double posy, double posz,double l1,double l2,double d,double bx,double by, double d1, double pos1, double h1,double d2, double pos2, double h2, double d3, double pos3, double h3, double d4, double pos4, double h4, double d5, double pos5, double h5) {
		if (show[3]==1) Kessel(quad,3.100,1.760,11.897,2.428,2,-1.732,1.9,0.577,10.501,0.462,0.924,7.344,0.193,0.924,4.534,0.193,1.5,5.990,0.193,0,0,0);

// void Fuhrerhaus(GLUquadric* quad,double posy,double l1,double l2,double h1,double h2,double h3,double b1,double b2,double x1,double y1,double x2,double y2,double x3,double y3,double x4,double y4) {
		if (show[4]==1) Fuhrerhaus(quad,1.617,1.760,2.695,1.040,1.078,0.5,3,2.5,0.154,0.154,0.847,0.963,0.963,0.154,1.656,0.847);

// void Zylinderblock(GLUquadric* quad,double r1,double r2,double h,double b,double posx,double posy,double posz,double l1,double l2) {
		if (show[5]==1) Zylinderblock(quad,0.231,0.3,0.654,0.15,12.195,1,0.75,1.0,1.3);

// void Triebwerk(GLUquadric* quad,double d,double posx,double posy,double posz,double l1,double x1,double x2,double x3,double x4,double x5,double x6,double x7,double x8,double x9, double x10,double x11,double x12) {
		if (show[6]==1) Triebwerk(quad,2,6.995,1.0,0.8,0.33,2.3,2.3,3.773,2.536,0.385,2.195,2.175,0.9,0.462,0.964,2.04,2.0);

// void Bleche(GLUquadric* quad,double posx,double posy,double posz,double l,double b,double h2, double l2,double b2,double x1,double x2,double wagl1,double wagh1,double wagl2,double wagh2,double luftx,double lufty,double luftz,double luftr,double luftl,double lx1,double ly1,double lz1,double lx2,double ly2) {
		if (show[7]==1) Bleche(quad,1.5,2.2,1.760,11.320,0.7,0.99,1.117,0.9,11.705,0.732,2.426,1.463,0.4,0.578,1.1,0.962,1.848,0.231,1.887,13.8,1.6,1.05,13.7,3.5);

// void TenderRahmen(GLUquadric* quad,double posx,double posy,double x1,double l1,double x2,double l2,double l3,double breite,double y,double dl1,double posx1,double dl2,double posx2) {
		if (show[8]==1) TenderRahmen(quad,0.160,1.1935,0.5,6.8915,1.386,4.235,0.424,1.3,0.5,2.1,2.275,2.1,6.075);

// void TenderRadsatz(GLUquadric* quad,double y,int s,double posx,double x1,double x2,double x3,double x4,double x5) {
		if (show[9]==1) TenderRadsatz(quad,1.0,9,0.16,1.165,1.9,1.9,1.9,0);

// void Tender(GLUquadric* quad,double posx,double posy,double x1,double h1,double l1,double x2,double h2,double l2,double h3,double l3,double ly1,double lz1,double ly2) {
		if (show[10]==1) Tender(quad,0.160,1.1935,0.5,1.848,6.8915,1.386,1.117,4.235,0.963,0.424,1.4,0.8,2.5);

// void Fenster(GLUquadric* quad, double posy, double l1, double l2, double h1, double h2, double h3, double b1, double b2, double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4) {
		if (show[4]==1) Fenster(quad,1.617,1.760,2.695,1.040,1.078,0.5,3,2.5,0.154,0.154,0.847,0.963,0.963,0.154,1.656,0.847);
		if (clip==1) glDisable(GL_CLIP_PLANE1);
	glPopMatrix();
	gluDeleteQuadric(quad);
}